@@ -99,9 +99,8 @@ class JobCluster(object):
 
         self._jobs = list()
 
-        self._jinja_env =  jinja2.Environment(loader=jinja2.PackageLoader('obob_condor', 'jinja2_templates'),
-                                              trim_blocks=True, lstrip_blocks=True)
-
+        self._jinja_env = jinja2.Environment(loader=jinja2.PackageLoader('obob_condor', 'jinja2_templates'),
+                                             trim_blocks=True, lstrip_blocks=True)
 
     def add_job(self, job, *args, **kwargs):
         """
@@ -156,8 +155,8 @@ class JobCluster(object):
 
         self._remove_notrunning_jobs()
         self._prepare_jobs_directory()
-        runner_fname = self._generate_runner_file()
-        submit_fname = self._generate_submit_file(runner_fname)
+        self._generate_runner_file()
+        submit_fname = self._generate_submit_file()
 
         submit_dict = dict()
         queue_glob = None
@@ -170,7 +169,7 @@ class JobCluster(object):
                     queue_glob = line.rsplit(' ', 1)[1]
 
         for idx, cur_file in enumerate(sorted(glob.glob(queue_glob))):
-            with open(os.path.join(self.condor_output_folder, 'log', 'out.%d' % (idx,)), 'wt') as out,\
+            with open(os.path.join(self.condor_output_folder, 'log', 'out.%d' % (idx,)), 'wt') as out, \
                     open(os.path.join(self.condor_output_folder, 'log', 'err.%d' % (idx,)), 'wt') as err:
                 this_args = submit_dict['arguments'].replace('$(filename)', os.path.abspath(cur_file))[2:-1].split(' ')
 
@@ -189,8 +188,8 @@ class JobCluster(object):
 
         self._remove_notrunning_jobs()
         self._prepare_jobs_directory()
-        runner_fname = self._generate_runner_file()
-        submit_fname = self._generate_submit_file(runner_fname)
+        self._generate_runner_file()
+        submit_fname = self._generate_submit_file()
 
         if do_submit:
             output = subprocess.check_output([self._condor_submit, submit_fname], stderr=subprocess.STDOUT, cwd=self.working_directory)
@@ -207,20 +206,18 @@ class JobCluster(object):
 
         self._jobs = new_job_list
 
-    def _generate_submit_file(self, runner_fname):
+    def _generate_submit_file(self):
 
         template = self._jinja_env.get_template(self._submit_template)
         submit_context = {
-            'python_bin': self.python_bin,
-            'runner': self._obob_condor_runner,
+            'executable': self.executable,
+            'arguments': self.arguments,
             'jobs_dir': self.condor_output_folder,
             'required_mem': int(self.required_ram / 1024 / 1024),
             'request_cpus': self.request_cpus,
             'owner': self.owner,
             'uuid': str(uuid.uuid4()),
-            'runner_fname': runner_fname,
             'python_home': self.python_home,
-            'singularity_image': self._singularity_image
         }
 
         submit_rendered = template.render(submit_context)
@@ -238,12 +235,9 @@ class JobCluster(object):
         }
 
         runner_rendered = template.render(runner_context)
-        runner_fname = os.path.join(self.condor_output_folder, 'condor', 'runner.py')
-        with open(runner_fname, 'wt') as runner_file:
+        with open(self.runner_filename, 'wt') as runner_file:
             runner_file.write(runner_rendered)
 
-        return runner_fname
-
     def _prepare_jobs_directory(self):
         final_jobs_dir = None
         if self.inc_jobsdir:
@@ -313,8 +307,8 @@ class JobCluster(object):
             python_bin = sys.executable
             if not python_bin.startswith('/mnt/obob/'):
                 logging.warning('Your current python distribution is not on /mnt/obob.\n'
-                                    'Using the default one in /mnt/obob/obob_mne now.\n'
-                                    'If you want to use a different one, please specify it explicitly in the constructor.')
+                                'Using the default one in /mnt/obob/obob_mne now.\n'
+                                'If you want to use a different one, please specify it explicitly in the constructor.')
 
                 python_bin = self._default_python_env
 
@@ -326,6 +320,29 @@ class JobCluster(object):
 
         self._python_bin = python_bin
 
+    @property
+    def singularity_command(self):
+        return '/usr/bin/singularity'
+
+    @property
+    def singularity_arguments(self):
+        return f'exec -S /home/{self.owner} -B /var/lib/condor -B /mnt --no-home --writable-tmpfs {self._singularity_image} ' \
+               f'{self.python_bin} {self.runner_filename}'
+
+    @property
+    def executable(self):
+        if self._singularity_image is not None:
+            return self.singularity_command
+        else:
+            return self.python_bin
+
+    @property
+    def arguments(self):
+        if self._singularity_image is not None:
+            return self.singularity_arguments
+        else:
+            return self.runner_filename
+
     @property
     def python_home(self):
         return os.path.dirname(os.path.dirname(self.python_bin))
@@ -355,6 +372,10 @@ class JobCluster(object):
     def n_jobs(self):
         return len(self._jobs)
 
+    @property
+    def runner_filename(self):
+        return os.path.join(self.condor_output_folder, 'condor', 'runner.py')
+
 
 class PermuteArgument(object):
     """
